(function() {
  var mongodb;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __slice = Array.prototype.slice, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  };
  mongodb = require('mongodb');
  exports.Connection = (function() {
    function Connection(options) {
      options || (options = {});
      this.ensureConnection(options);
      this.expires = options.expires || 60 * 60 * 1000;
      this.timeout = options.timeout || 10 * 1000;
    }
    Connection.prototype.ensureConnection = function(opt) {
      var server;
      this.queue = [];
      server = new mongodb.Server(opt.host || '127.0.0.1', opt.port || 27017);
      return new mongodb.Db(opt.db, server, {}).open(__bind(function(err, db) {
        return db.collection('mojo', __bind(function(err, collection) {
          var fn, _i, _len, _ref;
          this.mojo = collection;
          if (this.queue) {
            _ref = this.queue;
            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
              fn = _ref[_i];
              fn(collection);
            }
          }
          delete this.queue;
          return collection.ensureIndex([['queue'], ['expires'], ['owner']], function() {});
        }, this));
      }, this));
    };
    Connection.prototype.exec = function(fn) {
      return this.queue && this.queue.push(fn) || fn(this.mojo);
    };
    Connection.prototype.clear = function(queue, callback) {
      return this.exec(function(mojo) {
        return mojo.remove({
          queue: queue
        }, callback);
      });
    };
    Connection.prototype.enqueue = function() {
      var args, callback, expires, queue, _i;
      queue = arguments[0], args = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), callback = arguments[_i++];
      expires = new Date(new Date().getTime() + this.expires);
      return this.exec(function(mojo) {
        return mojo.insert({
          queue: queue,
          expires: expires,
          args: args
        }, callback);
      });
    };
    Connection.prototype.next = function(queue, owner, callback) {
      var now, timeout;
      now = new Date;
      timeout = new Date(now.getTime() + this.timeout);
      return this.exec(function(mojo) {
        return mojo.findAndModify({
          queue: queue,
          expires: {
            $gt: now
          },
          owner: null
        }, 'expires', {
          $set: {
            timeout: timeout,
            owner: owner
          }
        }, {
          "new": 1
        }, callback);
      });
    };
    Connection.prototype.complete = function(doc, callback) {
      return this.exec(function(mojo) {
        return mojo.findAndModify({
          _id: doc._id
        }, 'expires', {}, {
          remove: 1
        }, callback);
      });
    };
    Connection.prototype.release = function(doc, callback) {
      return this.exec(function(mojo) {
        return mojo.findAndModify({
          _id: doc._id
        }, 'expires', {
          $unset: {
            timeout: 1,
            owner: 1
          }
        }, {
          "new": 1
        }, callback);
      });
    };
    Connection.prototype.cleanup = function(queue, callback) {
      return this.exec(function(mojo) {
        return mojo.update({
          timeout: {
            $lt: new Date
          }
        }, {
          $unset: {
            timeout: 1,
            owner: 1
          }
        }, {
          multi: 1
        }, callback);
      });
    };
    return Connection;
  })();
  exports.Template = (function() {
    function Template(worker, doc) {
      this.worker = worker;
      this.doc = doc;
    }
    Template.prototype.invoke = function() {
      try {
        return this.perform.apply(this, this.doc.args);
      } catch (err) {
        return this.complete(err);
      }
    };
    Template.prototype.perform = function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      throw new Error('Yo, you need to implement me!');
    };
    Template.prototype.complete = function(err) {
      return this.worker.complete(err, this.doc);
    };
    return Template;
  })();
  exports.Worker = (function() {
    __extends(Worker, require('events').EventEmitter);
    function Worker(connection, templates, options) {
      this.connection = connection;
      this.templates = templates;
      options || (options = {});
      this.name = [require('os').hostname(), process.pid].join(':');
      this.timeout = options.timeout || 1000;
      this.workers = options.workers || 3;
      this.pending = 0;
    }
    Worker.prototype.poll = function() {
      var template;
      if (this.pending >= this.workers) {
        return this.sleep();
      }
      template = this.templates.shift();
      this.templates.push(template);
      return this.connection.next(template.name, this.name, __bind(function(err, doc) {
        if (err != null) {
          this.emit('error', err);
        } else if (doc != null) {
          ++this.pending;
          new template(this, doc).invoke();
          process.nextTick(__bind(function() {
            return this.poll();
          }, this));
        }
        return this.sleep();
      }, this));
    };
    Worker.prototype.sleep = function() {
      if (this.pollTimeout) {
        clearTimeout(this.pollTimeout);
      }
      return this.pollTimeout = setTimeout(__bind(function() {
        this.pollTimeout = null;
        return this.poll();
      }, this), this.timeout);
    };
    Worker.prototype.complete = function(err, doc) {
      var cb;
      cb = __bind(function() {
        --this.pending;
        return this.poll();
      }, this);
      if (err != null) {
        return this.connection.release(doc, cb);
      } else {
        return this.connection.complete(doc, cb);
      }
    };
    return Worker;
  })();
}).call(this);
